var outerWidth = 1060;
var outerHeight = 500;
var margin = { left: 80, top: 8, right: 150, bottom: 150 };
var barPadding = 0.2;

var xColumn = 'country';
var xColumnTooltip = 'country';
var yColumn = 'population';
var colorColumn = 'religion';
var layerColumn = colorColumn;

var hoveredColorValue;
var hoveredStrokeColor = 'black';

var innerWidth = outerWidth - margin.left - margin.right;
var innerHeight = outerHeight - margin.top - margin.bottom;

var svg = d3.select('body')
  .append('svg')
  .attr('width', outerWidth)
  .attr('height', outerHeight);
var g = svg.append('g')
  .attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');

var baseBarLayer = g.append('g');
var overlayRect = g.append('g')
  .append('rect')
  .attr('width', innerWidth)
  .attr('height', innerHeight)
  .attr('fill', 'none')
  .style('pointer-events', 'none');

var foregroundBarLayer = g.append('g');

var xAxisG = g.append('g')
  .attr('class', 'x axis')
  .attr('transform', 'translate(0,' + innerHeight + ')');
var yAxisG = g.append('g')
  .attr('class', 'y axis');
var colorLegendG = g.append('g')
  .attr('class', 'color-legend')
  .attr('transform', 'translate(635, 0)');

var xScale = d3.scale.ordinal()
  .rangeBands([0, innerWidth], barPadding);
var yScale = d3.scale.linear()
  .range([innerHeight, 0]);
var colorScale = d3.scale.category10();

var tipNumberFormat = d3.format(',');
var tip = d3.tip()
  .attr('class', 'd3-tip')
  .offset([60, 0])
  .html(function(d) {
    return [
      d[colorColumn],
      ' Population: ',
      d[yColumn]
    ].join('');
  });
g.call(tip);

var siFormat = d3.format('s');
var customTickFormat = function(d) {
  return siFormat(d).replace('G', 'B');
};

var xAxis = d3.svg
    .axis()
    .scale(xScale)
    .orient('bottom')               
    .outerTickSize(0);
var yAxis = d3.svg.axis()
  .scale(yScale)
  .orient('left')
  .ticks(5)
  .tickFormat(customTickFormat)
  .outerTickSize(0);

var colorLegend = d3.legend.color()
  .scale(colorScale)
  .shapePadding(6.24)
  .shapeWidth(25)
  .shapeHeight(25)
  .labelOffset(10);

function render(data) {
  var nested = d3.nest()
    .key(function(d) {
      return d[layerColumn];
    })
    .entries(data);

  var stack = d3.layout.stack()
    .y(function(d) {
      return d[yColumn];
    })
    .values(function(d) {
      return d.values;
    });

  var layers = stack(nested);

  var maxY = d3.max(layers, function(layer) {
    return d3.max(layer.values, function(d) {
      return d.y0 + d.y;
    });
  });
  xScale.domain(layers[0].values.map(function(d) {
    return d[xColumn];
  }));

  yScale.domain([0, maxY]);

  colorScale.domain(layers.map(function(layer) {
    return layer.key;
  }));

  xAxisG
      .call(xAxis)
        .selectAll("text")	
        .style("text-anchor", "end")
        .attr("dx", "-.8em")
        .attr("dy", ".15em")
        .attr("transform", function(d) {
            return "rotate(-65)" 
            });
    yAxisG.call(yAxis);

  renderBars(baseBarLayer, layers);

  if (hoveredColorValue) {
    setOverlayTransparency(0.7);
    var filteredLayers = layers.filter(function(layer) {
      return layer.key === hoveredColorValue;
    });
    renderBars(foregroundBarLayer, filteredLayers, layers);
  } else {
    setOverlayTransparency(0.0);
    renderBars(foregroundBarLayer, []);
  }

  colorLegendG.call(colorLegend);
  colorLegendG.selectAll('text').attr('y', 3);

  listenForHover(colorLegendG.selectAll('rect'), data);
  listenForHover(colorLegendG.selectAll('text'), data);
}

function renderBars(g, layers, masterLayers) {
  var layerGs = g.selectAll('.layer').data(layers);
  layerGs.enter().append('g').attr('class', 'layer');
  layerGs.exit().remove();

  var maxLayers = masterLayers || layers;
  var maxX = d3.max(maxLayers, function(layer) {
    return d3.max(layer.values, function(d) {
      return d.y0 + d.y;
    });
  });

  layerGs.style('fill', function(d) {
    return colorScale(d.key);
  });

  var bars = layerGs.selectAll('rect').data(function(d) {
    return d.values;
  });
  bars.enter().append('rect')
    .on('mouseover', function(d) {
      tip.show(d);
      d3.select('.d3-tip').style('left', '200px');
    })
    .on('mouseout', tip.hide);
  bars.exit().remove();
  bars.attr('x', function(d) {
      return xScale(d[xColumn]);
    })
    .attr('y', function(d) {
      return yScale(d.y0 + d.y);
    })
    .attr('width', xScale.rangeBand())
    .attr('height', function(d) {
      return yScale(d.y0) - yScale(d.y0 + d.y);
    });
}

function listenForHover(selection, data) {
  selection.on('mouseover', function(d) {
      hoveredColorValue = d;
      render(data);
    })
    .on('mouseout', function(d) {
      hoveredColorValue = null;
      render(data);
    })
    .style('cursor', 'pointer');
}

function setOverlayTransparency(alpha) {
  overlayRect
    .transition().duration(400)
    .attr('fill', 'rgba(255, 255, 255, ' + alpha + ')');
}

function type(d) {
  d.population = +d.population;
  return d;
}

const dataUrl = 'Data.csv';

d3.csv(dataUrl, type, render);
